__version__ = "0.0.21"
__authors__ = "Merey Ramazanova, Silvio Giancola"
__authors_username__ = "ramazam, giancos"
__author_email__ = "merey.ramazanova@kaust.edu.sa"
__github__ = 'https://github.com/meryusha/seeds_labeler/'
